# -*- coding: utf-8 -*-

from imio.schedule.workflow.freeze_task import FreezeTaskWorkflowAdaptation

import logging

logger = logging.getLogger('imio.schedule: migrations')


def migrate_to_18(context):
    """ """
    logger = logging.getLogger('imio.schedule: migrate to 1.8')
    logger.info("starting migration steps")

    logger.info("add freeze state to task workflow")
    adaptation = FreezeTaskWorkflowAdaptation()
    adaptation.patch_workflow('task_workflow')

    logger.info("reinstalling imio.schedule done!")
    logger.info("migration done!")



# -*- coding: utf-8 -*-

from datetime import date

from imio.schedule.content.logic import CreationTaskLogic
from imio.schedule.content.logic import TaskLogic
from imio.schedule.interfaces import ICondition
from imio.schedule.interfaces import ICreationCondition
from imio.schedule.interfaces import IEndCondition
from imio.schedule.interfaces import IFreezeCondition
from imio.schedule.interfaces import IMacroTaskCreationCondition
from imio.schedule.interfaces import IMacroTaskEndCondition
from imio.schedule.interfaces import IMacroTaskFreezeCondition
from imio.schedule.interfaces import IMacroTaskStartCondition
from imio.schedule.interfaces import IMacroTaskThawCondition
from imio.schedule.interfaces import IStartCondition
from imio.schedule.interfaces import IThawCondition

from zope.interface import implements


class Condition(TaskLogic):
    """
    Base class for TaskConfig conditions.
    """

    implements(ICondition)


class CreationCondition(CreationTaskLogic):
    """
    Creation condition of a AutomatedTask.
    """

    implements(ICreationCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class StartCondition(Condition):
    """
    Start condition of a AutomatedTask.
    """

    implements(IStartCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class EndCondition(Condition):
    """
    End condition of a AutomatedTask.
    """

    implements(IEndCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class FreezeCondition(Condition):
    """
    Freeze condition of a AutomatedTask.
    """

    implements(IFreezeCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class ThawCondition(Condition):
    """
    Thaw condition of a AutomatedTask.
    """

    implements(IThawCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class MacroTaskCreationCondition(CreationCondition):
    """
    Creation condition of a AutomatedMacroTask.
    """

    implements(IMacroTaskCreationCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class MacroTaskStartCondition(Condition):
    """
    Start condition of a AutomatedMacroTask.
    """

    implements(IMacroTaskStartCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class MacroTaskEndCondition(Condition):
    """
    End condition of a AutomatedMacroTask.
    """

    implements(IMacroTaskEndCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class MacroTaskFreezeCondition(Condition):
    """
    Freeze condition of a AutomatedMacroTask.
    """

    implements(IMacroTaskFreezeCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class MacroTaskThawCondition(Condition):
    """
    Thaw condition of a AutomatedMacroTask.
    """

    implements(IMacroTaskThawCondition)

    def evaluate(self):
        """
        To override.
        evaluate if the condition is True or False
        """


class CreateIfSubtaskCanBeCreated(MacroTaskCreationCondition):
    """
    Return True if at least one substask can be created.
    """

    def evaluate(self):
        """
        """


class CreateIfAllSubtasksCanBeCreated(MacroTaskCreationCondition):
    """
    Return True if all substasks can be created.
    """

    def evaluate(self):
        """
        """


class StartIfAnySubtaskStarted(MacroTaskStartCondition):
    """
    Return True if at least one substask started.
    """

    def evaluate(self):
        """
        """


class StartIfAllSubtasksStarted(MacroTaskStartCondition):
    """
    Return True if all substasks started.
    """

    def evaluate(self):
        """
        """


class RecurrencyCondition(Condition):

    def __init__(self, task_container, task_config):
        self.task_container = task_container
        self.task_config = task_config


class NoRecurencyCondition(RecurrencyCondition):

    def evaluate(self):
        return False


class TaskDueDateReachedCondition(EndCondition):
    """
    Return True if the task delay is overdue.
    """

    def evaluate(self):
        due_date = self.task.due_date
        today = date.today()
        due_date_reached = today >= due_date
        return due_date_reached


class MacroTaskDueDateReachedCondition(MacroTaskEndCondition):
    """
    Return True if the task delay is overdue.
    """

    def evaluate(self):
        due_date = self.task.due_date
        today = date.today()
        due_date_reached = today >= due_date
        return due_date_reached



# -*- coding: utf-8 -*-

from imio.schedule.content.subform_context_choice import SubFormContextChoice
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.deserializer.dxfields import DefaultFieldDeserializer
from plone.restapi.interfaces import IFieldDeserializer
from z3c.form.interfaces import IObjectFactory
from zope.component import adapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import Interface
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.schema.interfaces import IObject
from zope.schema.interfaces import IVocabularyFactory


@implementer(IFieldDeserializer)
@adapter(IObject, IDexterityContent, IBrowserRequest)
class ObjectDeserializer(DefaultFieldDeserializer):
    def __call__(self, value):
        if not self.field.schema:
            return self._default_call(value)
        adapter_name = "{0}.{1}".format(
            self.field.schema.__module__,
            self.field.schema.__name__,
        )
        factory_adapter = queryMultiAdapter(
            (Interface, Interface, Interface, Interface),
            IObjectFactory,
            name=adapter_name,
        )
        if not factory_adapter:
            return self._default_call()
        for fieldname in self.field.schema:
            field = self.field.schema[fieldname]
            if isinstance(field, SubFormContextChoice):
                self._set_vocabulary(field)
        return factory_adapter.factory(**value)

    def _set_vocabulary(self, field):
        """Ensure that vocabularies have the right values
        to avoid errors during validation"""
        vocabulary_factory = getUtility(
            IVocabularyFactory,
            name=field.vocabulary_name,
        )
        field.vocabulary = vocabulary_factory(self.context)

    def _default_call(self, value):
        return super(ObjectDeserializer, self).__call__(value)


# -*- coding: utf-8 -*-

# task status
CREATION = 'schedule_task_created'
STARTED = 'schedule_task_started'
DONE = 'schedule_task_done'
FROZEN = 'schedule_task_frozen'

states_by_status = {
    CREATION: [
        'created',
        'to_assign'
    ],
    STARTED: [
        'to_do',
        'in_progress',
        'realized'
    ],
    DONE: [
        'closed'
    ],
    FROZEN: [
        'frozen'
    ]
}

status_by_state = {
    'created': CREATION,
    'to_assign': CREATION,
    'to_do': STARTED,
    'in_progress': STARTED,
    'realized': STARTED,
    'closed': DONE,
    'frozen': FROZEN
}

task_types = ['task', 'AutomatedTask', 'AutomatedMacroTask']



# -*- coding: utf-8 -*-

from datetime import date

from imio.schedule.testing import ExampleScheduleIntegrationTestCase
from imio.schedule.config import CREATION, STARTED, DONE, status_by_state

from plone import api

from Products.ATContentTypes.interfaces import IATFolder


class TestUtils(ExampleScheduleIntegrationTestCase):
    """
    Test all methods of utils.py module.
    """

    def test_get_all_schedule_configs(self):
        """
        Test the method get_all_schedule_configs.
        """
        from imio.schedule.utils import get_all_schedule_configs

        self.assertEquals(
            [self.empty_schedule_config, self.schedule_config],
            get_all_schedule_configs()
        )

    def test_get_task_configs(self):
        """
        Test the method get_task_configs.
        """
        from imio.schedule.utils import get_task_configs

        expected_UIDS = [task_config.UID() for task_config in self.schedule_config.get_all_task_configs()]
        folder = self.portal.config
        task_configs = get_task_configs(folder)
        task_config_UIDS = [task_config.UID() for task_config in task_configs]
        self.assertEqual(set(task_config_UIDS), set(expected_UIDS))

    def test_get_container_open_tasks(self):
        """
        Test the method get_container_open_tasks.
        """
        from imio.schedule.utils import get_container_open_tasks

        task_state = api.content.get_state(self.task)
        self.assertEqual(status_by_state[task_state] in [CREATION, STARTED])
        expected = [self.task]
        open_tasks = get_container_open_tasks(self.task_container)
        self.assertEqual(expected, open_tasks)

    def test_end_all_open_tasks(self):
        """
        Test the method end_all_open_tasks.
        """
        from imio.schedule.utils import end_all_open_tasks

        open_tasks = end_all_open_tasks(self.task_container)
        end_all_open_tasks(self.task_container)
        for task in open_tasks:
            self.assertEqual(status_by_state[api.content.get_state()], DONE)

    def test_tuple_to_interface(self):
        """
        Should turn a tuple ('interface.module.path', 'Interface') into
        Interface class.
        """
        from imio.schedule.utils import tuple_to_interface

        expected_interface = IATFolder
        interface_tuple = ('Products.ATContentTypes.interfaces.folder', 'IATFolder')
        interface = tuple_to_interface(interface_tuple)
        self.assertEqual(interface, expected_interface)

    def test_interface_to_tuple(self):
        """
        Should turn an Interface class into a tuple:
        ('interface.module.path', 'Interface')
        """
        from imio.schedule.utils import interface_to_tuple

        expected_tuple = ('Products.ATContentTypes.interfaces.folder', 'IATFolder')
        interface_tuple = interface_to_tuple(IATFolder)
        self.assertEqual(interface_tuple, expected_tuple)


def TestWorkingDaysCalendar(ExampleScheduleIntegrationTestCase):

    def test_is_working_day(self):
        from imio.schedule.utils import WorkingDaysCalendar
        calendar = WorkingDaysCalendar()
        # Basic holiday
        self.assertTrue(calendar.is_working_day(date(2017, 1, 1)))
        # Test holiday from CalendarExtraHolidays utility
        self.assertTrue(calendar.is_working_day(date(2017, 2, 1)))



# -*- coding: utf-8 -*-

from dateutil.relativedelta import relativedelta

from imio.schedule.content.delay import CalculationDefaultDelay
from imio.schedule.content.task import IAutomatedTask
from imio.schedule.testing import ExampleScheduleFunctionalTestCase
from imio.schedule.testing import MacroTaskScheduleFunctionalTestCase

from mock import Mock

from plone import api

from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent


class TestTaskCreation(ExampleScheduleFunctionalTestCase):
    """
    Test task creation with different changes of TaskContainer.
    """

    def setUp(self):
        super(TestTaskCreation, self).setUp()
        self._adapter_computed_due_date = CalculationDefaultDelay.compute_due_date

    def tearDown(self):
        CalculationDefaultDelay.compute_due_date = self._adapter_computed_due_date
        super(TestTaskCreation, self).tearDown()

    def test_task_creation_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task should created automatically depending on start conditions
        and starting_states.
        """
        empty_task_container = self.empty_task_container
        msg = "so far, no task should have been created"
        self.assertEquals(len(empty_task_container.objectValues()), 0, msg)

        # simulate modification
        notify(ObjectModifiedEvent(empty_task_container))

        created = empty_task_container.objectValues()
        created = created and created[0]

        # a task should have been created
        msg = "A task should have been created"
        self.assertTrue(created, msg)

        msg = "The object created should have been a Task but is {}".format(
            created.portal_type
        )
        self.assertTrue(IAutomatedTask.providedBy(created), msg)

    def test_task_creation_on_container_workflow_modification(self):
        """
        When changing state of a contentype scheduled with a ScheduleConfig
        Task should created automatically depending on start conditions
        and starting_states.
        """
        empty_task_container = self.empty_task_container
        msg = "so far, no task should have been created"
        self.assertEquals(len(empty_task_container.objectValues()), 0, msg)

        # do workflow change
        api.content.transition(empty_task_container, transition='submit')
        api.content.transition(empty_task_container, transition='retract')

        created = empty_task_container.objectValues()
        created = created and created[0]

        # a task should have been created
        msg = "A task should have been created"
        self.assertTrue(created, msg)

        msg = "The object created should have been a Task but is {}".format(
            created.portal_type
        )
        self.assertTrue(IAutomatedTask.providedBy(created), msg)

    def test_task_creation_on_container_creation(self):
        """
        When creating a contentype scheduled with a ScheduleConfig
        Task should created automatically depending on start conditions
        and starting_states.
        """
        new_task_container = api.content.create(
            type='Folder',
            id='new_task_container',
            container=self.portal
        )

        created = new_task_container.objectValues()
        created = created and created[0]

        # a task should have been created
        msg = "A task should have been created"
        self.assertTrue(created, msg)

        msg = "The object created should have been a Task but is {}".format(
            created.portal_type
        )
        self.assertTrue(IAutomatedTask.providedBy(created), msg)

    def test_assigned_user_is_set_on_created_task(self):
        """
        Check that the assigned user is set on an
        automatically created task.
        """
        msg = 'default assigned user should have been admin'
        self.assertEquals(self.task.assigned_user, 'admin', msg)

    def test_due_date_is_set_on_created_task(self):
        """
        Check that the computed due date is set on an
        automatically created task.
        """
        msg = 'default du date should have been today + 10 days'
        due_date = self.task.due_date
        expected_date = self.task_container.creation_date
        delay = relativedelta(days=self.task_config.additional_delay)
        expected_date = expected_date.asdatetime().date() + delay
        self.assertEquals(due_date, expected_date, msg)

    def test_schedule_config_UID_is_set_on_created_task(self):
        """
        Check that schedule_config_UID attribute is set on an
        automatically created task.
        """
        task = self.task
        schedule_config = self.schedule_config
        msg = "schedule_config_UID attribute should have been set"
        self.assertEquals(task.schedule_config_UID, schedule_config.UID(), msg)

    def test_task_config_UID_is_set_on_created_task(self):
        """
        Check that task_config_UID attribute is set on an
        automatically created task.
        """
        task = self.task
        task_config = self.task_config
        msg = "task_config_UID attribute should have been set"
        self.assertEquals(task.task_config_UID, task_config.UID(), msg)


class TestTaskStarting(ExampleScheduleFunctionalTestCase):
    """
    Test task starting with different changes of TaskContainer.
    """

    def test_task_starting_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task should started automatically depending on start conditions
        and starting_states.
        """
        task_container = self.task_container
        task = self.task

        # put the task container on 'pending' state to match 'starting states'
        api.content.transition(task_container, transition='submit')
        # reopen the task to be sure it was not closed before the container
        # modification
        if api.content.get_state(task) == 'to_do':
            api.content.transition(task, 'back_in_to_assign')
        msg = "The task should not be started yet ! (for the sake of the test)"
        self.assertNotEquals(api.content.get_state(task), 'to_do', msg)

        # simulate modification
        notify(ObjectModifiedEvent(task_container))

        # the task should have been started
        msg = "The task should have been started"
        self.assertEquals(api.content.get_state(task), 'to_do', msg)

    def test_task_starting_on_container_workflow_modification(self):
        """
        When changing state a contentype scheduled with a ScheduleConfig
        Task should started automatically depending on start conditions
        and starting_states.
        """
        task_container = self.task_container
        task = self.task

        msg = "The task should not be closed yet ! (for the sake of the test)"
        self.assertNotEquals(api.content.get_state(task), 'to_do', msg)

        # do workflow change
        api.content.transition(task_container, transition='submit')

        # the task should have been started
        msg = "The task should have been started"
        self.assertEquals(api.content.get_state(task), 'to_do', msg)


class TestTaskUpdate(ExampleScheduleFunctionalTestCase):
    """
    Test task update with different changes of TaskContainer.
    """

    def test_update_due_date_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task due date should be updated.
        """
        task_container = self.task_container
        task = self.task
        old_due_date = task.due_date

        # set an additional delay of 42 days on the task config
        CalculationDefaultDelay.calculate_delay = Mock(return_value=42)
        msg = "The task due date should not have changed"
        self.assertEquals(task.due_date, old_due_date)

        # simulate modification
        notify(ObjectModifiedEvent(task_container))

        msg = "The task due date should have been updated"
        self.assertNotEquals(task.due_date, old_due_date, msg)

    def test_reindex_due_date_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task due date should be updated and reindexed.
        """
        task_container = self.task_container
        task = self.task
        old_due_date = task.due_date

        # set an additional delay of 42 days on the task config
        CalculationDefaultDelay.calculate_delay = Mock(return_value=42)
        msg = "The task due date should not have changed"
        self.assertEquals(task.due_date, old_due_date)

        # simulate modification
        notify(ObjectModifiedEvent(task_container))

        catalog = api.portal.get_tool('portal_catalog')
        msg = 'catalog should not find anything with old due date'
        task_brain = catalog(due_date=old_due_date, UID=task.UID())
        self.assertFalse(task_brain, msg)
        msg = 'new due date should have been reindexed'
        task_brain = catalog(due_date=task.due_date, UID=task.UID())
        self.assertTrue(task_brain, msg)


class TestMacroTaskUpdate(MacroTaskScheduleFunctionalTestCase):
    """
    Test task update with different changes of TaskContainer.
    """

    def tearDown(self):
        api.content.transition(obj=self.macro_task, to_state='created')

    def test_update_recurrence_handler(self):
        """
        When modifying a contenttype the recurrence should be evaluated
        """
        transitions = ['do_to_assign', 'do_realized', 'do_closed']
        self.macrotask_config.activate_recurrency = True
        self.macrotask_config.recurrence_conditions = self.macrotask_config.creation_conditions
        self.macrotask_config.recurrence_states = ('private', )

        self.subtask_config.activate_recurrency = True
        self.subtask_config.recurrence_conditions = self.subtask_config.creation_conditions
        self.subtask_config.recurrence_states = ('private', )

        for transition in transitions:
            api.content.transition(obj=self.macro_task, transition=transition)
        notify(ObjectModifiedEvent(self.task_container))

        self.assertTrue('TASK_test_macrotaskconfig-1' in self.task_container)
        macro_task = self.task_container['TASK_test_macrotaskconfig-1']
        self.assertTrue('TASK_test_subtaskconfig' in macro_task)
        for transition in transitions:
            api.content.transition(obj=macro_task, transition=transition)


class TestTaskEnding(ExampleScheduleFunctionalTestCase):
    """
    Test task ending with different changes of TaskContainer.
    """

    def test_task_ending_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task should ended automatically depending on end conditions
        and ending_states.
        """
        task_container = self.task_container
        task = self.task

        # put the task container on 'published' state to match 'ending states'
        api.content.transition(task_container, transition='publish')
        # reopen the task to be sure it was not closed before the container
        # modification
        if api.content.get_state(task) == 'closed':
            api.content.transition(task, 'back_in_realized')
        msg = "The task should not be closed yet ! (for the sake of the test)"
        self.assertNotEquals(api.content.get_state(task), 'closed', msg)

        # simulate modification
        notify(ObjectModifiedEvent(task_container))

        # the task should have been ended
        msg = "The task should have been ended"
        self.assertEquals(api.content.get_state(task), 'closed', msg)

    def test_task_ending_on_container_workflow_modification(self):
        """
        When changing state a contentype scheduled with a ScheduleConfig
        Task should ended automatically depending on end conditions
        and ending_states.
        """
        task_container = self.task_container
        task = self.task

        msg = "The task should not be closed yet ! (for the sake of the test)"
        self.assertNotEquals(api.content.get_state(task), 'closed', msg)

        # do workflow change
        api.content.transition(task_container, transition='publish')

        # the task should have been ended
        msg = "The task should have been ended"
        self.assertEquals(api.content.get_state(task), 'closed', msg)


class TestTaskFreezing(ExampleScheduleFunctionalTestCase):
    """
    Test task freezing with different changes of TaskContainer.
    """

    def test_task_freezing_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task should froze automatically depending on freeze conditions
        and freezing_states.
        """
        task_container = self.task_container
        task = self.task
        task_config = self.task_config
        task_config.ending_states = ['published']
        task_config.freeze_states = ['private']

        msg = "The task should not be closed or frozen yet ! (for the sake of the test)"
        self.assertNotEquals(task.get_state(), 'closed', msg)
        self.assertNotEquals(task.get_state(), 'frozen', msg)
        msg = "The task container should  be in private state"
        self.assertEquals(api.content.get_state(task_container), 'private', msg)

        # simulate modification
        notify(ObjectModifiedEvent(task_container))

        # the task should have been frozen
        msg = "The task should have been frozen"
        self.assertEquals(api.content.get_state(task), 'frozen', msg)

    def test_task_freezing_on_container_workflow_modification(self):
        """
        When changing state a contentype scheduled with a ScheduleConfig
        Task should froze automatically depending on freeze conditions
        and freezing_states.
        """
        task_container = self.task_container
        task = self.task
        task_config = self.task_config
        task_config.ending_states = ['private']
        task_config.freeze_states = ['published']

        msg = "The task should not be closed or frozen yet ! (for the sake of the test)"
        self.assertNotEquals(task.get_state(), 'closed', msg)
        self.assertNotEquals(task.get_state(), 'frozen', msg)

        # do workflow change
        api.content.transition(task_container, transition='publish')

        # the task should have been frozen
        msg = "The task should have been frozen"
        self.assertEquals(task.get_state(), 'frozen', msg)


class TestTaskThawing(ExampleScheduleFunctionalTestCase):
    """
    Test task thawing with different changes of TaskContainer.
    """

    def test_task_thawing_on_container_modification(self):
        """
        When modifying a contentype scheduled with a ScheduleConfig
        Task should froze automatically depending on thaw conditions
        and thawing_states.
        """
        task_container = self.task_container
        task = self.task
        task_config = self.task_config
        task_config.ending_states = ['published']
        task_config.thaw_states = ['private']

        task_original_state = task.get_state()
        msg = "The task original state should not be frozen! (for the sake of the test)"
        self.assertNotEquals(task_original_state, 'frozen', msg)
        msg = "The task container should  be in private state"
        self.assertEquals(api.content.get_state(task_container), 'private', msg)

        # freeze task
        task_config.freeze_task(task)
        self.assertEquals(task.get_state(), 'frozen', msg)
        # simulate modification
        notify(ObjectModifiedEvent(task_container))

        # the task should have been thawed
        msg = "The task should have been thawed"
        self.assertEquals(api.content.get_state(task), task_original_state, msg)

    def test_task_thawing_on_container_workflow_modification(self):
        """
        When changing state a contentype scheduled with a ScheduleConfig
        Task should froze automatically depending on thaw conditions
        and thawing_states.
        """
        task_container = self.task_container
        task = self.task
        task_config = self.task_config
        task_config.freeze_states = ['private']
        task_config.thaw_states = ['published']

        task_original_state = task.get_state()
        msg = "The task original state should not be frozen! (for the sake of the test)"
        self.assertNotEquals(task_original_state, 'frozen', msg)

        # freeze task
        task_config.freeze_task(task)
        self.assertEquals(task.get_state(), 'frozen', msg)
        # do workflow change
        api.content.transition(task_container, transition='publish')

        # the task should have been thawed
        msg = "The task should have been thawed"
        self.assertEquals(task.get_state(), task_original_state, msg)



# -*- coding: utf-8 -*-

from DateTime import DateTime

from plone import api

from imio.schedule.config import STARTED
from imio.schedule.config import status_by_state
from imio.schedule.interfaces import IDefaultTaskGroup
from imio.schedule.interfaces import IDefaultTaskUser
from imio.schedule.interfaces import IMacroTaskStartDate
from imio.schedule.interfaces import IStartDate
from imio.schedule.interfaces import ITaskLogic

from zope.interface import implements


class CreationTaskLogic(object):
    """
    Base class for any object adapting a task container into
    some task logic (condition, user assigment, due date..)
    called during the task craetion.
    """

    implements(ITaskLogic)

    def __init__(self, task_container, task_config):
        self.task_container = task_container
        self.task_config = task_config


class TaskLogic(object):
    """
    Base class for any object adapting a task container and a task
    into some logic (condition, user assigment, due date..).
    """

    implements(ITaskLogic)

    def __init__(self, task_container, task):
        self.task_container = task_container
        self.task = task
        self.task_config = task.get_task_config()


class StartDate(TaskLogic):
    """
    Base class for TaskConfig due dates.
    """

    implements(IStartDate)

    def start_date(self):
        """
        To override.
        Compute a due date from task_container
        then return it.
        """


class MacroTaskStartDate(StartDate):
    """
    Base class for TaskConfig due dates.
    """

    implements(IMacroTaskStartDate)

    def start_date(self):
        """
        To override.
        Compute a due date from task_container
        then return it.
        """


class TaskStartingDate(StartDate):
    """
    Return the date when the task started.
    """

    def start_date(self):
        """
        Return the date when the task started.
        """
        history = self.task.workflow_history.values()[0]
        for state_history in history:
            if status_by_state[state_history.get('review_state')] is STARTED:
                return state_history['time']


class SubtaskHighestDueDate(MacroTaskStartDate):
    """
    Return the highest due date of the subtasks.
    """

    def start_date(self):
        """
        Return the highest due date of the subtasks.
        """
        subtasks = self.task.get_subtasks()
        if not subtasks:
            return None
        due_dates = [DateTime(str(t.due_date)) for t in subtasks if t.due_date]
        return due_dates and max(due_dates) or None


class AssignTaskUser(TaskLogic):
    """
    Base class for adapters adapting a TaskContainer to return a user to
    assign to its tasks.
    Register adapters inheriting this class in the products using
    imio.schedule and override 'user_id' method.
    """
    implements(IDefaultTaskUser)

    def user_id(self):
        """
        To override.
        """


class AssignCurrentUser(AssignTaskUser):
    """
    Return the current connected user to assign it as default assigned
    user of a new AutomatedTask.
    """

    def user_id(self):
        """
        Return the id of the current user.
        """
        user = api.user.get_current()
        user_id = user.getUserName()
        return user_id


class AssignOwnerOfLastSimilarTask(AssignTaskUser):
    """
    Return the current connected user to assign it as default assigned
    user of a new AutomatedTask.
    """

    def user_id(self):
        """
        Return the id of the current user.
        """
        user = api.user.get_current()
        user_id = user.getUserName()
        return user_id


class AssignTaskGroup(TaskLogic):
    """
    Base class for adapters adapting a TaskContainer to return a group to
    assign to its tasks.
    Register adapters inheriting this class in the products using
    imio.schedule and override 'group_id' method.
    """
    implements(IDefaultTaskGroup)

    def group_id(self):
        """
        To override.
        """



# -*- coding: utf-8 -*-

from DateTime import DateTime

from imio.schedule.interfaces import IMacroTaskStartDate
from imio.schedule.interfaces import IStartDate
from imio.schedule.testing import ExampleScheduleIntegrationTestCase
from imio.schedule.testing import MacroTaskScheduleIntegrationTestCase

from zope.component import queryMultiAdapter
from zope.component import getMultiAdapter


class TestTaskLogicIntegration(ExampleScheduleIntegrationTestCase):
    """
    Test some logic for tasks.
    """

    def test_task_starting_date_registration(self):
        """
        Content types voc factory should be registered as a named utility.
        """
        logic_name = 'schedule.start_date.task_starting_date'

        duedate_adapter = queryMultiAdapter(
            (self.task_container, self.task),
            IStartDate,
            name=logic_name
        )
        self.assertTrue(duedate_adapter)

    def test_task_starting_date_logic(self):
        """
        Check if the SubtaskHighestDueDate adapter return the highest
        due date amongst all the subtasks of a AutomatedMacroTask.
        """
        task_config = self.task_config
        task = self.task
        logic_name = 'schedule.start_date.task_starting_date'

        duedate_adapter = getMultiAdapter(
            (self.task_container, task),
            IStartDate,
            name=logic_name
        )

        task_config.start_task(task)
        due_date = duedate_adapter.start_date()
        expected_date = task.workflow_history.values()[0][-1]['time']
        self.assertEquals(due_date, expected_date)


class TestMacroTaskLogicIntegration(MacroTaskScheduleIntegrationTestCase):
    """
    Test MacroTaskConfig methods.
    """

    def test_subtask_highest_due_date_registration(self):
        """
        Content types voc factory should be registered as a named utility.
        """
        logic_name = 'schedule.start_date.subtask_highest_due_date'

        duedate_adapter = queryMultiAdapter(
            (self.task_container, self.macro_task),
            IMacroTaskStartDate,
            name=logic_name
        )
        self.assertTrue(duedate_adapter)

    def test_subtask_highest_due_date_logic(self):
        """
        Check if the SubtaskHighestDueDate adapter return the highest
        due date amongst all the subtasks of a AutomatedMacroTask.
        """
        logic_name = 'schedule.start_date.subtask_highest_due_date'

        duedate_adapter = getMultiAdapter(
            (self.task_container, self.macro_task),
            IMacroTaskStartDate,
            name=logic_name
        )

        highest_due_date = duedate_adapter.start_date()
        expected_date = DateTime(str(self.sub_task.due_date))
        self.assertEquals(highest_due_date, expected_date)



# encoding: utf-8

from collective.eeafaceted.z3ctable.columns import BaseColumn

from plone import api

from imio.schedule import _
from imio.schedule.config import CREATION
from imio.schedule.config import DONE
from imio.schedule.config import STARTED
from imio.schedule.content.task import IAutomatedMacroTask
from imio.schedule.dashboard.interfaces import IDisplayTaskStatus
from imio.schedule.dashboard.interfaces import ISimpleDisplayTaskStatus
from imio.schedule.dashboard.interfaces import IStatusColumn
from imio.schedule.interfaces import IToIcon
from imio.schedule.utils import close_or_past_date

from zope.component import queryAdapter
from zope.component import queryMultiAdapter
from zope.interface import implements
from zope.i18n import translate


def due_date_extra_classes(due_date, review_state):
    extra_classes = ""

    if review_state != 'closed' and due_date and due_date.year != 9999:
        orange_limit = api.portal.get_registry_record(
            'imio.schedule.interfaces.IDueDateSettings.color_orange_x_days_before_due_date',
            default=None
        )
        red_limit = api.portal.get_registry_record(
            'imio.schedule.interfaces.IDueDateSettings.color_red_x_days_before_due_date',
            default=None
        )
        if red_limit is not None and close_or_past_date(due_date, by_days=red_limit):
            extra_classes += " red_close_due_date"
        elif orange_limit is not None and close_or_past_date(due_date, by_days=orange_limit):
            extra_classes += " orange_close_due_date"

    return extra_classes


class DueDateColumn(BaseColumn):
    """ Due date column for schedule listings."""

    def renderCell(self, item):
        due_date = item.due_date
        if not due_date or due_date.year == 9999:
            return u'\u221E'

        return due_date.strftime('%d/%m/%Y')

    def getCSSClasses(self, item):
        """Returns a CSS class specific to current content."""
        cssClasses = super(DueDateColumn, self).getCSSClasses(item)

        cssClasses["td"] += due_date_extra_classes(item.due_date, item.review_state)

        return cssClasses


class AssignedUserColumn(BaseColumn):
    """ display licence address in SearchResultTable """

    def renderCell(self, item):
        username = item.assigned_user
        groupname = item.assigned_group

        user = username and api.user.get(username)
        username = user and user.getProperty('fullname').decode('utf-8') or username
        assigned = username
        if groupname:
            to_icon = queryAdapter(item, IToIcon)
            group = api.group.get(groupname)
            if to_icon:
                icon_url = to_icon.get_icon_url()
                groupname = '<img src="{}">'.format(icon_url)
            else:
                groupname = group and group.getProperty('title').decode('utf-8') or groupname
                groupname = '({})'.format(groupname)

            assigned = u'{user} {group}'.format(
                user=username,
                group=groupname
            )

        return assigned


class StatusColum(BaseColumn):
    """
    Column displaying the status of the tasks and its subtasks if it has any.
    """
    implements(IStatusColumn)

    sort_index = -1
    display_status_interface = IDisplayTaskStatus
    escape = False

    def renderHeadCell(self):
        """Override rendering of head of the cell to include jQuery
           call to initialize annexes menu and to show the 'more/less details' if we are listing items."""
        # activate necessary javascripts
        if not self.header_js:
            # avoid problems while concataining None and unicode
            self.header_js = u''
        self.header_js += u'<script type="text/javascript">' + \
            u'$("#task_status a").prepOverlay({subtype: "ajax"});' + \
            '</script>'
        return super(StatusColum, self).renderHeadCell()

    def renderCell(self, item):
        """
        """
        status = ''
        task = item.getObject()

        adapter = queryMultiAdapter(
            (self, task, api.portal.getRequest()),
            self.display_status_interface
        )
        if adapter:
            status = adapter.render()
        return status


class SimpleStatusColumn(StatusColum):
    """
    Used for task listing on a single task container.
    """
    display_status_interface = ISimpleDisplayTaskStatus


class SimpleTaskStatusDisplay(object):
    """
    Adpater, adapting a task and returning some html
    table cell displaying its status.
    """

    implements(IDisplayTaskStatus)

    def __init__(self, column, task, request):
        self.task = task
        self.request = request

    def render(self):
        task = self.task
        status = u'<span class="simple_task">&nbsp&nbsp&nbsp</span>'
        link = '<a class="link-overlay" href="{task_url}/@@item_view">{status}</a>'.format(
            task_url=task.absolute_url(),
            status=status
        )
        status_display = '<span id="task_status">{}</span>'.format(link)
        return status_display


class TaskStatusDisplay(object):
    """
    Adpater, adapting a task and returning some html
    table cell displaying its status.
    """

    implements(IDisplayTaskStatus)

    def __init__(self, column, task, request):
        self.task = task
        self.request = request

    def render(self):
        task = self.task
        return self.display_task_status(task, with_subtasks=IAutomatedMacroTask.providedBy(task))

    def display_task_status(self, task, with_subtasks=False):
        """
        By default just put a code colour of the state of the task.
        """
        css_class = 'schedule_{}'.format(api.content.get_state(task))
        css_level = 'term-level-{}'.format(self.task.level())
        status = u'<span class="{css_level}"><span class="{css_class}">&nbsp&nbsp&nbsp</span></span>'.format(
            css_level=css_level,
            css_class=css_class,
        )
        if task.get_status() in [CREATION, STARTED]:
            viewname = '{}{}_status'.format(
                (not with_subtasks) and 'simple_' or '',
                task.get_status() is CREATION and 'start' or 'end',
            )
            status = '<a class="link-overlay" href="{task_url}/@@{view}">{status}</a>'.format(
                task_url=task.absolute_url(),
                view=viewname,
                status=status
            )
        status = '<span id="task_status">{}</span>'.format(status)
        return status


class MacroTaskStatusDisplay(TaskStatusDisplay):
    """
    Adapts a macro task and return some html table cell
    displaying the status of all subtasks that need to be done.
    """

    def render(self):
        """
        """
        all_subtasks = self.task.get_last_subtasks()
        subtasks_to_do = [task for task in all_subtasks if task.get_status() != DONE]
        if not subtasks_to_do:
            return self.display_task_status(self.task)

        rows = [
            u'<tr><th class="subtask_status_icon">{icon}</th>\
            <th i18n:translate="">{subtask}</th>\
            <th i18n:translate="">{due_date}</th></tr>'.format(
                icon=self.display_task_status(self.task, with_subtasks=bool(subtasks_to_do)),
                subtask=translate(_('Subtask'), context=self.request),
                due_date=translate(_('Due date'), context=self.request),
            ),
        ]
        for task in subtasks_to_do:
            title = task.Title()
            display_subtasks = IAutomatedMacroTask.providedBy(task)
            status_icon = u'<td class="subtask_status_icon">{status}</td>'.format(
                status=self.display_task_status(task, with_subtasks=display_subtasks),
            )
            status_title = u'<td class="subtask_status_title">{title}</td>'.format(
                title=title.decode('utf-8'),
            )
            date = task.due_date
            due_date = date.year == 9999 and u'\u221E' or date.strftime('%d/%m/%Y')
            extra_css_classes = due_date_extra_classes(
                task.due_date,
                api.content.get_state(task),
            )
            due_date = u'<td class="subtask_status_date{}">{}</td>'.format(extra_css_classes, due_date)
            row = u'<tr>{icon}{title}{due_date}</tr>'.format(
                icon=status_icon,
                title=status_title,
                due_date=due_date
            )
            rows.append(row)

        status_display = u'<table class=subtask_status_table>{}</table>'.format(''.join(rows))
        return status_display



# -*- coding: utf-8 -*-

from eea.facetednavigation.layout.interfaces import IFacetedLayout

from imio.dashboard.browser.facetedcollectionportlet import Assignment
from collective.eeafaceted.collectionwidget.utils import _updateDefaultCollectionFor

from imio.schedule.config import CREATION
from imio.schedule.config import STARTED
from imio.schedule.config import states_by_status
from imio.schedule.content.task import IAutomatedTask
from imio.schedule.interfaces import IToTaskConfig
from imio.schedule.interfaces import ICalendarExtraHolidays

from plone import api
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.constants import CONTEXT_CATEGORY

from zope.annotation import IAnnotations
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import getUtilitiesFor
from zope.schema.vocabulary import SimpleVocabulary
from workalendar.europe import Belgium

import datetime
import importlib


def get_all_schedule_configs():
    """
    Return all the ScheduleConfig of the site.
    """
    # nested import to avoid recursive imports
    from imio.schedule.content.schedule_config import IScheduleConfig

    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog(object_provides=IScheduleConfig.__identifier__)
    configs = [brain.getObject() for brain in brains]

    return configs


def get_task_configs(task_container, descending=False):
    """
    Return all the task configs to check for the given context
    providing ITaskContainer.
    """
    config_adapters = getAdapters((task_container,), IToTaskConfig)
    task_configs = [adapter.task_config for name, adapter in config_adapters]
    ordering = descending and 1 or -1
    task_configs = sorted(task_configs, key=lambda cfg: ordering * cfg.level())

    return task_configs


def get_container_open_tasks(task_container):
    """
    Return all the open tasks of a container.
    """
    states = states_by_status[CREATION] + states_by_status[STARTED]
    open_tasks = get_container_tasks(task_container, states)
    return open_tasks


def end_all_open_tasks(task_container):
    """
    End all open tasks of a container without checking any condition.
    """
    tasks_to_end = get_container_open_tasks(task_container)
    for task in tasks_to_end:
        task._end()
    return tasks_to_end


def get_container_tasks(task_container, states=[]):
    """
    Return all the tasks of a container.
    """
    open_tasks = []
    to_explore = [task_container]
    while to_explore:
        current = to_explore.pop()
        if IAutomatedTask.providedBy(current):
            if not states:
                open_tasks.append(current)
            elif api.content.get_state(current) in states:
                open_tasks.append(current)
        if hasattr(current, 'objectValues'):
            to_explore.extend(current.objectValues())
    return open_tasks


def tuple_to_interface(interface_tuple):
    """
    Turn a tuple of strings:
    ('interface.module.path', 'Interface')
    into an Interface class.
    """
    module_path, interface_name = interface_tuple
    interface_module = importlib.import_module(module_path)
    interface_class = getattr(interface_module, interface_name)

    return interface_class


def interface_to_tuple(interface):
    """
    Turn an Interface class into a tuple of strings:
    ('interface.module.path', 'Interface')
    """
    return (interface.__module__, interface.__name__)


def set_schedule_view(folder, faceted_config_path, schedule_configs, default_collection=None):
    """
    Boilerplate code to set up the schedule view on a folderish context.
    """

    if type(schedule_configs) not in [list, tuple]:
        schedule_configs = [schedule_configs]

    _set_faceted_view(folder, faceted_config_path, schedule_configs, default_collection)
    _set_collection_portlet(folder)


def _set_faceted_view(folder, faceted_config_path, schedule_configs, default_collection=None):
    """
    """
    annotations = IAnnotations(folder)
    key = 'imio.schedule.schedule_configs'
    annotations[key] = [cfg.UID() for cfg in schedule_configs]

    subtyper = folder.restrictedTraverse('@@faceted_subtyper')
    if not subtyper.is_faceted:
        subtyper.enable()
        folder.restrictedTraverse('@@faceted_settings').toggle_left_column()
        IFacetedLayout(folder).update_layout('faceted-table-items')
        folder.unrestrictedTraverse('@@faceted_exportimport').import_xml(
            import_file=open(faceted_config_path)
        )

    default_collection = default_collection or schedule_configs[0].dashboard_collection
    _updateDefaultCollectionFor(folder, default_collection.UID())


def _set_collection_portlet(folder):
    """
    """
    # block parent portlets
    manager = getUtility(IPortletManager, name='plone.leftcolumn')
    blacklist = getMultiAdapter((folder, manager), ILocalPortletAssignmentManager)
    blacklist.setBlacklistStatus(CONTEXT_CATEGORY, True)

    # assign collection portlet
    manager = getUtility(IPortletManager, name='plone.leftcolumn', context=folder)
    mapping = getMultiAdapter((folder, manager), IPortletAssignmentMapping)
    if 'schedules' not in mapping.keys():
        mapping['schedules'] = Assignment('schedules')


def dict_list_2_vocabulary(dict_list):
    """dict_list_2_vocabulary
    Converts a dictionary list to a SimpleVocabulary

    :param dict_list: dictionary list
    """
    terms = []
    for item in dict_list:
        for key in sorted([k for k in item]):
            terms.append(SimpleVocabulary.createTerm(
                key, str(key), item[key]))
    return SimpleVocabulary(terms)


def round_to_weekday(date, weekday):
    direction = weekday / abs(weekday)  # -1 => past, +1 => future
    weekday = abs(weekday) - 1
    days_delta = weekday - date.weekday()
    if days_delta * direction < 0:
        days_delta += 7 * direction
    return date + datetime.timedelta(days_delta)


def close_or_past_date(date, by_days=7):
    """
    Checks if a date is getting close (or already past)

    :type date: datetime.date
    :type by_days: int
    """
    today = datetime.date.today()
    limit_date = date - datetime.timedelta(days=by_days)
    return today >= limit_date


class WorkingDaysCalendar(Belgium):

    def __init__(self, *args, **kwargs):
        super(WorkingDaysCalendar, self).__init__(*args, **kwargs)
        self.working_days = self._get_working_days()

    def is_working_day(self, date, *args, **kwargs):
        result = super(WorkingDaysCalendar, self).is_working_day(
            date, *args, **kwargs)
        if result is True:
            result = date.isoweekday() in self.working_days
        return result

    def get_calendar_holidays(self, year):
        result = super(WorkingDaysCalendar, self).get_calendar_holidays(year)
        for name, utility in getUtilitiesFor(ICalendarExtraHolidays):
            result += utility.get_holidays(year)
        return result

    def _get_working_days(self):
        """Return the working days configured in the registry"""
        matching = {
            'monday': 1,
            'tuesday': 2,
            'wednesday': 3,
            'thursday': 4,
            'friday': 5,
            'saturday': 6,
            'sunday': 7,
        }
        days = api.portal.get_registry_record(
            'imio.schedule.interfaces.ISettings.working_days'
        ) or []
        return [matching[d] for d in days]



# -*- coding: utf-8 -*-

from imio.schedule.content.logic import StartDate


class ContainerCreationDate(StartDate):
    """
    Test StartDate returning the creation date of the task container.
    """

    def start_date(self):
        return self.task_container.creation_date



# -*- coding: utf-8 -*-

from zope.interface import Interface
from z3c.form.form import Form
from z3c.form import button
from z3c.form import field
from Products.statusmessages.interfaces import IStatusMessage
from plone.z3cform.layout import FormWrapper
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from imio.schedule import _


class ICloseTask(Interface):
    """ """


class CloseTaskForm(Form):
    fields = field.Fields(ICloseTask)
    ignoreContext = True

    @button.buttonAndHandler(_(u'Confirm'))
    def handleApply(self, action):
        messages = IStatusMessage(self.request)
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            messages.addStatusMessage(self.status, type="error")
            return
        self.context._end()
        self.context.reindex_parent_tasks(idxs=['is_solvable_task'])
        self.status = _(u'Task Closed')


class CloseTaskView(FormWrapper):
    form = CloseTaskForm
    index = ViewPageTemplateFile('templates/close_task.pt')



# -*- coding: utf-8 -*-

from plone import api

from imio.schedule.testing import ExampleScheduleIntegrationTestCase


class TestIndexes(ExampleScheduleIntegrationTestCase):
    """
    Test custom indexes.
    """

    def test_schedule_config_UID_index(self):
        """
        Check is schedule config UID is indexed in tasks.
        """
        schedule_config = self.schedule_config
        task = self.task
        catalog = api.portal.get_tool('portal_catalog')

        task_brains = catalog(schedule_config_UID=schedule_config.UID())
        msg = "should have found at least one indexed task"
        self.assertEquals(len(task_brains), 1, msg)
        self.assertEquals(task_brains[0].getObject(), task)

    def test_task_config_UID_index(self):
        """
        Check is task config UID is indexed in tasks.
        """
        task_config = self.task_config
        task = self.task
        catalog = api.portal.get_tool('portal_catalog')

        task_brains = catalog(task_config_UID=task_config.UID())
        msg = "should have found at least one indexed task"
        self.assertEquals(len(task_brains), 1, msg)
        self.assertEquals(task_brains[0].getObject(), task)



# -*- coding: utf-8 -*-

from z3c.form.object import FactoryAdapter
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

from imio.schedule.content.task_config import ICreationConditionSchema
from imio.schedule.content.task_config import IEndConditionSchema
from imio.schedule.content.task_config import IFreezeConditionSchema
from imio.schedule.content.task_config import IMacroCreationConditionSchema
from imio.schedule.content.task_config import IMacroEndConditionSchema
from imio.schedule.content.task_config import IMacroFreezeConditionSchema
from imio.schedule.content.task_config import IMacroRecurrenceConditionSchema
from imio.schedule.content.task_config import IMacroStartConditionSchema
from imio.schedule.content.task_config import IMacroThawConditionSchema
from imio.schedule.content.task_config import IRecurrenceConditionSchema
from imio.schedule.content.task_config import IStartConditionSchema
from imio.schedule.content.task_config import IThawConditionSchema


class BaseConditionObject(object):
    """
    Base class for condition objects.
    """

    __name__ = ''
    __parent__ = None

    def __init__(self, condition=None, operator=None, display_status=True):
        if condition is not None:
            self.__dict__['condition'] = condition
        if operator is not None:
            self.__dict__['operator'] = operator
        self.__dict__['display_status'] = display_status

    def getId(self):
        return self.__name__ or ''


class CreationConditionObject(BaseConditionObject):
    implements(ICreationConditionSchema)

    condition = FieldProperty(ICreationConditionSchema['condition'])
    operator = FieldProperty(ICreationConditionSchema['operator'])
    display_status = FieldProperty(ICreationConditionSchema['display_status'])


class CreationConditionAdapter(FactoryAdapter):
    factory = CreationConditionObject


class StartConditionObject(BaseConditionObject):
    implements(IStartConditionSchema)

    condition = FieldProperty(IStartConditionSchema['condition'])
    operator = FieldProperty(IStartConditionSchema['operator'])
    display_status = FieldProperty(IStartConditionSchema['display_status'])


class StartConditionAdapter(FactoryAdapter):
    factory = StartConditionObject


class EndConditionObject(BaseConditionObject):
    implements(IEndConditionSchema)

    condition = FieldProperty(IEndConditionSchema['condition'])
    operator = FieldProperty(IEndConditionSchema['operator'])
    display_status = FieldProperty(IEndConditionSchema['display_status'])


class EndConditionAdapter(FactoryAdapter):
    factory = EndConditionObject


class FreezeConditionObject(BaseConditionObject):
    implements(IFreezeConditionSchema)

    condition = FieldProperty(IFreezeConditionSchema['condition'])
    operator = FieldProperty(IFreezeConditionSchema['operator'])
    display_status = FieldProperty(IFreezeConditionSchema['display_status'])


class FreezeConditionAdapter(FactoryAdapter):
    factory = FreezeConditionObject


class ThawConditionObject(BaseConditionObject):
    implements(IThawConditionSchema)

    condition = FieldProperty(IThawConditionSchema['condition'])
    operator = FieldProperty(IThawConditionSchema['operator'])
    display_status = FieldProperty(IThawConditionSchema['display_status'])


class ThawConditionAdapter(FactoryAdapter):
    factory = ThawConditionObject


class MacroCreationConditionObject(BaseConditionObject):
    implements(IMacroCreationConditionSchema)

    condition = FieldProperty(IMacroCreationConditionSchema['condition'])
    operator = FieldProperty(IMacroCreationConditionSchema['operator'])
    display_status = FieldProperty(IMacroCreationConditionSchema['display_status'])


class MacroCreationConditionAdapter(FactoryAdapter):
    factory = MacroCreationConditionObject


class MacroStartConditionObject(BaseConditionObject):
    implements(IMacroStartConditionSchema)

    condition = FieldProperty(IMacroStartConditionSchema['condition'])
    operator = FieldProperty(IMacroStartConditionSchema['operator'])
    display_status = FieldProperty(IMacroStartConditionSchema['display_status'])


class MacroStartConditionAdapter(FactoryAdapter):
    factory = MacroStartConditionObject


class MacroEndConditionObject(BaseConditionObject):
    implements(IMacroEndConditionSchema)

    condition = FieldProperty(IMacroEndConditionSchema['condition'])
    operator = FieldProperty(IMacroEndConditionSchema['operator'])
    display_status = FieldProperty(IMacroEndConditionSchema['display_status'])


class MacroEndConditionAdapter(FactoryAdapter):
    factory = MacroEndConditionObject


class MacroFreezeConditionObject(BaseConditionObject):
    implements(IMacroFreezeConditionSchema)

    condition = FieldProperty(IMacroFreezeConditionSchema['condition'])
    operator = FieldProperty(IMacroFreezeConditionSchema['operator'])
    display_status = FieldProperty(IMacroFreezeConditionSchema['display_status'])


class MacroFreezeConditionAdapter(FactoryAdapter):
    factory = MacroFreezeConditionObject


class MacroThawConditionObject(BaseConditionObject):
    implements(IMacroThawConditionSchema)

    condition = FieldProperty(IMacroThawConditionSchema['condition'])
    operator = FieldProperty(IMacroThawConditionSchema['operator'])
    display_status = FieldProperty(IMacroThawConditionSchema['display_status'])


class MacroThawConditionAdapter(FactoryAdapter):
    factory = MacroThawConditionObject


class RecurrenceConditionObject(BaseConditionObject):
    implements(IRecurrenceConditionSchema)

    condition = FieldProperty(IRecurrenceConditionSchema['condition'])
    operator = FieldProperty(IRecurrenceConditionSchema['operator'])
    display_status = FieldProperty(IRecurrenceConditionSchema['display_status'])


class RecurrenceConditionAdapter(FactoryAdapter):
    factory = RecurrenceConditionObject


class MacroRecurrenceConditionObject(BaseConditionObject):
    implements(IMacroRecurrenceConditionSchema)

    condition = FieldProperty(IMacroRecurrenceConditionSchema['condition'])
    operator = FieldProperty(IMacroRecurrenceConditionSchema['operator'])
    display_status = FieldProperty(IMacroRecurrenceConditionSchema['display_status'])


class MacroRecurrenceConditionAdapter(FactoryAdapter):
    factory = MacroRecurrenceConditionObject



# -*- coding: utf-8 -*-

from imio.schedule.content.object_factories import CreationConditionObject
from imio.schedule.content.object_factories import EndConditionObject
from imio.schedule.content.object_factories import FreezeConditionObject
from imio.schedule.content.object_factories import MacroCreationConditionObject
from imio.schedule.content.object_factories import MacroEndConditionObject
from imio.schedule.content.object_factories import MacroFreezeConditionObject
from imio.schedule.content.object_factories import MacroRecurrenceConditionObject
from imio.schedule.content.object_factories import MacroStartConditionObject
from imio.schedule.content.object_factories import MacroThawConditionObject
from imio.schedule.content.object_factories import RecurrenceConditionObject
from imio.schedule.content.object_factories import StartConditionObject
from imio.schedule.content.object_factories import ThawConditionObject
from plone.restapi.interfaces import IJsonCompatible
from zope.component import adapter
from zope.interface import implementer


def base_converter(value):
    return value.__dict__


@adapter(CreationConditionObject)
@implementer(IJsonCompatible)
def creation_condition_converter(value):
    return base_converter(value)


@adapter(StartConditionObject)
@implementer(IJsonCompatible)
def start_condition_converter(value):
    return base_converter(value)


@adapter(EndConditionObject)
@implementer(IJsonCompatible)
def end_condition_converter(value):
    return base_converter(value)


@adapter(FreezeConditionObject)
@implementer(IJsonCompatible)
def freeze_condition_converter(value):
    return base_converter(value)


@adapter(ThawConditionObject)
@implementer(IJsonCompatible)
def thaw_condition_converter(value):
    return base_converter(value)


@adapter(RecurrenceConditionObject)
@implementer(IJsonCompatible)
def recurrence_condition_converter(value):
    return base_converter(value)


@adapter(MacroCreationConditionObject)
@implementer(IJsonCompatible)
def macro_creation_condition_converter(value):
    return base_converter(value)


@adapter(MacroStartConditionObject)
@implementer(IJsonCompatible)
def macro_start_condition_converter(value):
    return base_converter(value)


@adapter(MacroEndConditionObject)
@implementer(IJsonCompatible)
def macro_end_condition_converter(value):
    return base_converter(value)


@adapter(MacroFreezeConditionObject)
@implementer(IJsonCompatible)
def macro_freeze_condition_converter(value):
    return base_converter(value)


@adapter(MacroThawConditionObject)
@implementer(IJsonCompatible)
def macro_thaw_condition_converter(value):
    return base_converter(value)


@adapter(MacroRecurrenceConditionObject)
@implementer(IJsonCompatible)
def macro_recurrence_condition_converter(value):
    return base_converter(value)
